-- tpSet4
-- Set of Tiled Polygons with size 4

local function createTpSet4Procedural()
	-- Define the surrounding vertices
	local size = 4
	local vertices = {}
	for i = 1, size do
		local t1 = i - 1
		vertices[i] = {t1, 0}
		vertices[i + size] = {size, t1}
		local t2 = size - i + 1
		vertices[i + size * 2] = {t2, size}
		vertices[i + size * 3] = {0, t2}
	end
	print (#vertices .. ' vertices')

	local sides = {{},{},{},{}}
	for i = 1, size + 1 do
		table.insert(sides[1], i)
		table.insert(sides[2], i + size)
		table.insert(sides[3], i + 2 * size)
		table.insert(sides[4], i + 3 * size)
	end

	local corners = {{},{},{},{}}
	for i = 1, 2 * size + 1 do
		table.insert(corners[1], i)
		table.insert(corners[2], i + size)
		table.insert(corners[3], i + 2 * size)
		table.insert(corners[4], i + 3 * size)
	end

	local edges = {1, size + 1, 2 * size + 1, 3 * size + 1}
	local amount = #vertices

	local polygons = {}
	local polygonHash = {}

	local v2 = size + 1 -- 5
	local v3 = 2*size + 1 -- 9
	local v4 = 3*size + 1 -- 13

	-- first is from corner along side:
	for v1 = 1, size do

		

		local v_to = #vertices
		if v1 == 1 then
			v_to = v4-1
		end

		for v_last = v2+1, v_to do
			
			local polygon = {v1}

			if v_last > v2 then 
				table.insert (polygon, v2)
				if v_last > v3 then 
					table.insert (polygon, v3)
					if v_last > v4 then 
						table.insert (polygon, v4)
					end
				end
			end

			table.insert (polygon, v_last)

			table.insert(polygons, polygon)
		end

	end

	local tpSet4 = {
		side = size,
		vertices = vertices,
		polygons = polygons,
	}
	return tpSet4
end


------------------------
------------------------

local function showVertices (tpSet)
	local vertices = tpSet.vertices
	print ('#vertices:', #vertices)
	print ('vertices = {')
	for i, vertice in ipairs (vertices) do
		print ('	{'..table.concat(vertice, ',')..'}, --' .. i)
	end
	print ('}')
end

local function showPolygons (tpSet)
	local polygons = tpSet.polygons
	print ('#polygons:', #polygons)
	print ('polygons = {')
	for i, polygon in ipairs (polygons) do
		print ('	{'..table.concat(polygon, ',')..'}, --' .. i)
	end
	print ('}')
end

------------------------
------------------------

-- Usage:
local tpSet4 = createTpSet4Procedural()

showVertices (tpSet4)
showPolygons (tpSet4)

return tpSet4